//
//  Copyright (C) 1996 Dallas Semiconductor Corporation.
//  All rights Reserved. Printed in U.S.A.
//  This software is protected by copyright laws of
//  the United States and of foreign countries.
//  This material may also be protected by patent laws of the United States
//  and of foreign countries.
//  This software is furnished under a license agreement and/or a
//  nondisclosure agreement and may only be used or copied in accordance
//  with the terms of those agreements.
//  The mere transfer of this software does not imply any licenses
//  of trade secrets, proprietary technology, copyrights, patents,
//  trademarks, maskwork rights, or any other form of intellectual
//  property whatsoever. Dallas Semiconductor retains all ownership rights.
//
/*
        scratch.c

      This program prints the ROM data of all Buttons found on the parallel
   port, and reads the scratchpad of any DS1425 buttons found.
*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <mem.h>

#define TRUE  1
#define FALSE 0

#define READSCRATCH 0x69                    /* Read scratchpad command byte */
#define SP_SA       0xC0              /* Field specifier / starting address */

typedef unsigned char  uchar;
typedef unsigned short ushort;

uchar dowcheck(void);                            /* Access system functions */
uchar keyopen(void);
uchar keyclose(void);
uchar setup(uchar);
uchar next(void);
uchar first(void);
uchar ds_access(void);
uchar gndtest(void);
uchar databyte(uchar);
uchar *romdata(void);
uchar OverdriveOff(void);
uchar OverdriveOn(void);
uchar DOWReset(uchar);
uchar DOWBit(uchar);
uchar DOWByte(uchar);
void main(int argc, char *argv[])
{
	 float  Average;
    uchar  pn = 0, ch;                                   /* Parallel port number */
    uchar  *addr;                                  /* Holds ROM data pointer */
    uchar  NoButtons;
    int    i, j, BtnCount = 0, NumRounds = 0;
    int    loop_num;
    int    read_num;
    uchar  FRes, NRes;
    uchar  MStr[32];
    if (argc > 1)                /* Grab read_num parameter off command line */
      read_num = atoi(argv[1]);
    else
      read_num = 1;
    if (dowcheck())                     /* See if device driver is installed */
    {
        while ((++pn < 4) && setup(pn))                   /* Search all ports */
        {
	        if(keyopen())
	        {
	            loop_num = 0; NoButtons = FALSE;     /* Initialize loop counter */
	            while ((loop_num++ < read_num))
	            {
                   printf("\nCalling First\n");
                   i = first();
	                if (i)                   /* Look for first part on bus */
	                {
		                do
		                {
									BtnCount++;
	                        keyclose();  /* Release control of port between passes */
	                        keyopen();
                           printf(" \rRom Data: ");
		                     addr = romdata();           /* Get pointer to ROM data */
                           for (i = 0; i < 8; i++)            /* Display ROM data */
                           printf("%2.2X ", *(addr	+ i));
                           printf("\n");
                      }
                      while (next());              /* Look at all parts on port */
                   }
                   else
                   {
                        NoButtons = TRUE;
                        printf("No buttons found on port %d\n",pn);
                   }
     		   	    printf("\nNumber of Rounds = %d", loop_num);
						 printf("\nAverage Number Found = %f\n", (Average = (float)BtnCount/(float)loop_num));
               }

            }
            else
               printf("\nFAILED KEYOPEN\n");

            keyclose();
	    BtnCount = 0;
        }

    }
    else
        printf("DOW device driver not installed.\n");
}



