//
//  Copyright (C) 1996 Dallas Semiconductor Corporation. 
//  All rights Reserved. Printed in U.S.A.
//  This software is protected by copyright laws of
//  the United States and of foreign countries.
//  This material may also be protected by patent laws of the United States 
//  and of foreign countries.
//  This software is furnished under a license agreement and/or a
//  nondisclosure agreement and may only be used or copied in accordance
//  with the terms of those agreements.
//  The mere transfer of this software does not imply any licenses
//  of trade secrets, proprietary technology, copyrights, patents,
//  trademarks, maskwork rights, or any other form of intellectual
//  property whatsoever. Dallas Semiconductor retains all ownership rights.
//
//  Modifications: 7-16-96 Made it DS1410E compatible.
//                 1-02-97 Added check overdrive function and modified
//				keyopen and keyclose so they work with DS1410Es
#define INCL_DOSDEVICES
#define INCL_DOSPROCESS
#define INCL_DOSSEMAPHORES

#include <stdio.h>
#include <conio.h>
#include <os2.h>
#include <dos.h>
#include <string.h>

#define CATCODE  0x91
#define FUNCTION 0x01
#define BYTECODE 0x06
#define KEYOPEN  0x07
#define KEYCLOSE 0x08
#define TOGOD    0x09
#define DOCOMML  0x0A
#define BURNMSEC 0x0B
#define ODCHECK  0x0C
typedef unsigned int word;
typedef unsigned char uchar;
typedef unsigned long dword;

word fileattribute, file_open, i;
static uchar InitSem = TRUE;
static HMTX  DOWHandle1;
APIRET retcode;
HFILE dow_handle; 
ULONG actiontaken;
ULONG filesize=0;
PEAOP2 reserved;
uchar keychar[2];
uchar authflag = 0;
uchar kans = 0;
uchar *mbuf;
uchar count=0;
uchar lpt_num, setup_ok = 0, fail_next = 0;
uchar x, bits[65], romdta[8];
uchar access_good = 1, l0;
uchar Passthru = TRUE;

PVOID ParmList;
PVOID DataArea;
ULONG ParmLengthMax;
ULONG ParmLengthInOut;
ULONG DataLengthMax;
ULONG DataLengthInOut;

uchar keyopen();
uchar keyclose();
uchar setup(uchar p_n);
uchar next(void);
uchar first(void);
uchar ds_access();
uchar databyte(uchar br);
uchar *romdata();
uchar gndtest();
uchar dowcheck();
void OverdriveOff(void);
uchar OverdriveOn(void);

static uchar DOWByte(uchar byte_val);
static uchar DOWBit(uchar bit_val);
static uchar DOWReset(uchar p_n);
static void  CreateOrOpenSem(void);
static uchar romsearch(void);  //romsearch
static uchar ExitPassthru(void);
static uchar EnterPassthru(void);
static uchar ToggleOverdrive(void);
static void  TogglePassthru(void);
static uchar DS1481Present(void);
static uchar CheckOverdrive(void);

/*
//
//    For DOS and Windows, keyopen, keyclose and dowcheck perform no useful
// function. They are included only for portability. Other environments
// (such as NT, unix, etc) require the use of these functions to avoid 
// collisions on the 1-wire bus resulting from multiple processes attempting
// button communication.
//
*/
static void CreateOrOpenSem(void)
{
   if (DosOpenMutexSem("\\SEM32\\DOWSem0", &DOWHandle1) != 0)
       DosCreateMutexSem("\\SEM32\\DOWSem0", &DOWHandle1, 0, 0);
		
   InitSem = FALSE;

   return;
}

uchar keyopen()
{
APIRET rc;
   if (InitSem)
      CreateOrOpenSem();
   // if mutex semaphore is achieved, return TRUE

   if (!authflag)
   {
      if (DosRequestMutexSem(DOWHandle1, 20000))
      {
	 authflag = FALSE;
	 return authflag;
      }
      else
      {
	 InitSem = FALSE;
	 authflag = TRUE;

	 rc = DosOpen("DOWOS2$",
		 &dow_handle,
		 &actiontaken,
		 filesize,
		 FILE_SYSTEM | FILE_NORMAL,
		 OPEN_ACTION_OPEN_IF_EXISTS,
		 OPEN_SHARE_DENYNONE|OPEN_FLAGS_FAIL_ON_ERROR|OPEN_ACCESS_READWRITE,
		 reserved);
      }
   }
   if(authflag)
   {
		DOWReset(lpt_num);								 // this sets the port number
      ExitPassthru();
//		OverdriveOn();
	   if(CheckOverdrive())
{
         OverdriveOff();
		   printf("\nPART IS IN OVERDRIVE\n");
} 
   }
if(!authflag)
	printf("\nFAILED KEYOPEN\nAUTHFLAG = FALSE\n");
   return authflag;                              /* Return control status */
}

uchar keyclose()
{
   if (authflag)
   {
      EnterPassthru();
      // Release the mutex semophore
      DosReleaseMutexSem(DOWHandle1);
      authflag = 0;
      DosClose(dow_handle);
      DosSleep(100); /* Don't allow process to grab key(again) right away */
   }
   return 1;
}

static uchar DOWReset(uchar p_n)
{
   if (authflag)
   {
      keychar[0] = p_n + 2; keychar[1] = 0;

      DataLengthInOut = 0;
      DataLengthMax = 0;

      ParmLengthMax = 1;
      ParmLengthInOut = 1;

      retcode = DosDevIOCtl(dow_handle, CATCODE, FUNCTION, keychar,
			    ParmLengthMax, &ParmLengthInOut, NULL,
			    DataLengthMax, &DataLengthInOut);


      return keychar[0];
   }
   return 0;
}

uchar setup(uchar p_n)
{
   setup_ok = 0;
   if ((p_n > 0) && (p_n < 4))
   {
      lpt_num = p_n;
      setup_ok = fail_next = access_good = TRUE;
   }
   else
      lpt_num = 1; //p_n;
   return setup_ok;
}

static uchar DOWBit(uchar bit_val)
{
   if (authflag)
   {
      keychar[0] = bit_val;

      DataLengthInOut = 0;
      DataLengthMax = 0;

      ParmLengthMax = 1;
      ParmLengthInOut = 1;

      retcode = DosDevIOCtl(dow_handle, CATCODE, FUNCTION, keychar,
			    ParmLengthMax, &ParmLengthInOut, NULL,
			    DataLengthMax, &DataLengthInOut);


      return keychar[0];
   }
   return 0;
}

static uchar DOWByte(uchar byte_val)
{
   uchar i, brecv = 0;
   if (authflag)
   {
      /* call dowbit 8 times to transfer entire byte*/
      for (i = 0; i < 8; i++)
      {
    	brecv >>=1;
	    brecv |= DOWBit(byte_val & 1) ? 0x80 : 0; /* Smush bit into byte*/
	    byte_val >>= 1;
      }
      return brecv;
   }
   return 0;
}

static uchar romsearch(void)
{
   uchar i = 0, x = 0, ld = l0;
   uchar RomBit, Mask;

   // Reset DOW bus and send search command
   if (DOWReset(lpt_num))
      DOWByte(0xF0);

   // While not done and bus error
   while ((i++ < 64) && (x < 3))
   {
      // Get bit mask
      Mask = 1 << ((i - 1) % 8);

      RomBit = romdta[(i - 1) >> 3] & Mask ? TRUE : FALSE;

      // Send first read time slot
      x = DOWBit(TRUE) << 1;  
      // Send second read time slot
      x |= DOWBit(TRUE);      

      // Is there a disagreement in this bit position   
      if (!x)               
      {
         // Stay on old path or pick a new one ?    
         if (i >= ld)              
            RomBit = (i == ld);   // Send write 1 if at position of ld    

         // Save this value as temp last disagreement    
         if (!RomBit)         
            l0 = i;
      }
      else 
         RomBit = (x & 1) ^ 1;    // Get lsb of x and flip it    

      if (RomBit)              
         romdta[(i - 1) >> 3] |= Mask;          // Set bit in Rom Data byte   
      else
         romdta[(i - 1) >> 3] &= (Mask ^ 0xFF); //Clear bit in Rom Data byte  

      // Send write time slot   
      DOWBit(RomBit);                               
   }

   return (x == 3) ? 0 : 1 + (ld == l0);    

}

uchar next(void)
{
   uchar tr, xi, cntr;
   if (setup_ok && authflag)
   {
      if (fail_next)
      {
        fail_next = 0; l0 = 0;
      }
      else while ((tr = romsearch()) != 0)
      {
	 cntr = 0;
	 if (tr == 2)
	    fail_next = 1;
	 if (!romdta[0])
	    return FALSE;
	 access_good = TRUE;
	 return TRUE;
      }
   }
   return 0;
}

uchar first(void)
{
   l0 = 0;
   fail_next = FALSE;
   return next();
}

uchar ds_access()
{
   uchar xi;

   access_good = 0;
   if(authflag)
   {
      if (DOWReset(lpt_num) && setup_ok)
      {
	 DOWByte(0x55);
	 for (xi = 0; xi < 8; xi++) DOWByte(romdta[xi]);

	 access_good = 1;
      }
   }
   return access_good;
}

uchar databyte(uchar br)
{
   if (setup_ok & access_good & authflag) return DOWByte(br); else return br;
}

uchar *romdata()
{
   return romdta;
}

uchar gndtest()
{
   if (setup_ok & authflag)
      return TRUE;
   return 0;
}
/*
   See if DOWOS2 installed.
*/
uchar dowcheck()
{
   uchar retcode;

   retcode = DosOpen("DOWOS2$",
		      &dow_handle,
		      &actiontaken,
		      filesize,
		      FILE_SYSTEM | FILE_NORMAL,
		      OPEN_ACTION_OPEN_IF_EXISTS,
		      OPEN_SHARE_DENYNONE | OPEN_FLAGS_FAIL_ON_ERROR
		      | OPEN_ACCESS_READWRITE,
		      reserved);

   if (retcode) return 0;

   DosClose(dow_handle);                                /* Close driver */
   return 1;
}

void OverdriveOff(void)
{
   if (authflag)
   {
      // Turn overdrive off
      if (ToggleOverdrive())
	 ToggleOverdrive();
   }
}

uchar OverdriveOn(void)
{
   if (authflag)
   {
      if (!DOWReset(lpt_num))
	 return FALSE;
      // Put all parts in overdrive mode
      DOWByte(0x3C);
      return ToggleOverdrive() ? TRUE : ToggleOverdrive();
   }
   return FALSE;
}

static uchar ExitPassthru(void)
{
   uchar i = 0;
   Passthru = TRUE;

	if(authflag)
	{
   	while (Passthru && (i++ < 3))
   	{
      	TogglePassthru();
      	Passthru = !DS1481Present();
printf("\nNumPass = %d\n", i);
   	}
	}
   return Passthru;
}
static uchar EnterPassthru(void)
{
	if (authflag)
	{
   	TogglePassthru();
   	Passthru = TRUE;
   	return TRUE;
	}
	return FALSE;
}

static void TogglePassthru(void)
{
   uchar i;
   if (authflag)
   {
        for (i = 0; i < 4; i++)
            ToggleOverdrive();
        DosSleep(20);
    }
}
static uchar ToggleOverdrive(void)
{
   keychar[0] = TOGOD;

   DataLengthInOut = 0;
   DataLengthMax = 0;

   ParmLengthMax = 1;
   ParmLengthInOut = 1;

   retcode = DosDevIOCtl(dow_handle, CATCODE, FUNCTION, keychar,
			 ParmLengthMax, &ParmLengthInOut, NULL,
			 DataLengthMax, &DataLengthInOut);
   return keychar[0];
}
static uchar DS1481Present(void)
{
	keychar[0] = 0;
   if(authflag)
	{
		DOWReset(lpt_num);
   	keychar[0] = DOCOMML;
   	DataLengthInOut = 0;
   	DataLengthMax = 0;

   	ParmLengthMax = 1;
   	ParmLengthInOut = 1;
   	
		retcode = DosDevIOCtl(dow_handle, CATCODE, FUNCTION, keychar,
				 ParmLengthMax, &ParmLengthInOut, NULL,
			 	DataLengthMax, &DataLengthInOut);
	}
   return keychar[0];
}
static uchar CheckOverdrive(void)
{
	keychar[0] = 0;
   if(authflag)
	{
   	keychar[0] = ODCHECK;
   	DataLengthInOut = 0;
   	DataLengthMax = 0;

   	ParmLengthMax = 1;
   	ParmLengthInOut = 1;
   	
		retcode = DosDevIOCtl(dow_handle, CATCODE, FUNCTION, keychar,
				 ParmLengthMax, &ParmLengthInOut, NULL,
			 	DataLengthMax, &DataLengthInOut);
	}
   return keychar[0];
}
